/*
* @Author: José Elías Gutierrez
* @Date:   2017-04-10 15:43:31
* @Last Modified by:   José Elías Gutierrez
* @Last Modified time: 2017-04-20 16:52:07
*/

'use strict';

/**
* Ajax request to calculate and return price, and store booking session. Overrides plugin default function
* Make sure its called after wceb class is loaded
*/
var jebe = {};
jQuery(document).ready(function(){

	(function($) {
		jebe = {
			
			elements: {
				'start_hour'	: $('#start_hour'),
				'end_hour'		: $('#end_hour'),
				'inputStart'	: $('.datepicker_start').pickadate(),
				'inputEnd'	   	: $('.datepicker_end').pickadate(),
				'pickerStart'	: $('.datepicker_start').pickadate('picker'),
				'pickerEnd'		: $('.datepicker_end').pickadate('picker')
			},

			times: [],

			options: jebe_options,

			disableHolidays: function( dates, pickerStartItem, pickerEndItem, mode ){

				var toDisable = ebac.getDates( data.disabled_dates[variationId], mode );
				
				$.each(dates, function(index, value){

					var date = new Date(value + ' 00:00:00');
					var dateToDisable = [date.getFullYear(),date.getMonth(),date.getDate()];
					var time = date.getTime();
					dateToDisable['type'] = 'booked';

					toDisable['dates'].push( dateToDisable ); // Array of dates object (day mode)
					toDisable['times'].push( time );

				} );

				if ( toDisable.length <= 0 ) {
					pickerStartItem.disable = [];
					pickerEndItem.disable   = [];
					return false;
				}

				pickerStartItem.disable = ( mode === 'nights' ) ? dates.startDates : toDisable;

				// In "Nights" mode, end calendar dates have a one day offset
				pickerEndItem.disable = toDisable;

				this.times = toDisable;

				return false;
			},

		};

		jebe.elements.start_hour.on('change', function(){wceb.setPrice();});
		jebe.elements.end_hour.on('change', function(){wceb.setPrice();});

		jebe.elements.pickerStart.on('after_rendering', function(){

			$.each( jebe.times['times'], function( index, datePick ) {
				var $day = $(".picker__day[data-pick='"+datePick+"']");
				$day.addClass('picker__day--disabled');
			});
		});

		$('body').on( 'pickers_init', function( e, variation ) {

			if(jebe.options.allow_holiday_bookings !== 'on'){

				jebe.disableHolidays(jebe.options.holidays, jebe.elements.pickerStart, jebe.elements.pickerEnd, wceb.calcMode);

			}

			return false;

		});

		
		wceb.setPrice = function() {
			console.log('jebe override');
			this.product_id   = $('input[name=add-to-cart], button[name=add-to-cart]').val();
			this.variation_id = $('.variations_form').find('input[name=variation_id]').val();

			this.children = wceb.get.childrenIds();
			
			this.selectedDates = {};

			var format = $.fn.pickadate.defaults.format;

			// Start date
			this.selectedDates['startFormat'] = pickerStart.get('select', 'yyyy-mm-dd'), // yyyy-mm-dd
			this.selectedDates['start']       = pickerStart.get('select', format); // Nice format

			// End date
			this.selectedDates['endFormat'] = pickerEnd.get('select', 'yyyy-mm-dd'), // yyyy-mm-dd
			this.selectedDates['end']       = pickerEnd.get('select', format); // Nice format

			// WooCommerce Product Add-ons compatibility
			var additionalCost = wceb.get.additionalCosts( 'each' );

			var data = {
				action         : 'add_new_price',
				security       : $('input[name="_wceb_nonce"]').val(),
				product_id     : this.product_id,
				variation_id   : this.variation_id,
				children       : this.children,
				start          : this.selectedDates['start'],
				end            : this.selectedDates['end'],
				start_format   : this.selectedDates['startFormat'],
				end_format     : this.selectedDates['endFormat'],
				additional_cost: additionalCost,
				// Hours info
				start_hour	   : jebe.elements.start_hour.val(),
				end_hour	   : jebe.elements.end_hour.val(),
			};
			console.log(data);
			$('form.cart, form.bundle_form').fadeTo('400', '0.6').block({
				message: null,
				overlayCSS: {
					background: 'transparent',
					backgroundSize: '16px 16px',
					opacity: 0.6
				}
			});

			$.post( wceb_object.ajax_url, data, function( response ) {
				console.log(response);
				$('.woocommerce-error, .woocommerce-message').remove();
				this.fragments = response.fragments;
				this.errors    = response.errors;

				// If error, reset pickers
				if ( this.errors ) {

					$.each( this.errors, function( key, value ) {
						$( key ).replaceWith( value );
					});

					wceb.pickers.init();
					pickerStart.render();
					pickerEnd.render();

					// Unblock
					$('form.cart, form.bundle_form').stop(true).css('opacity', '1').unblock();

					return false;

				}

				if ( this.fragments ) {

					$.each( this.fragments, function( key, value ) {
						$( key ).replaceWith( value );
					});

					// Get quantity selected
					if ( $('.cart').find('input[name="quantity"]').size() ) {
						var qty_selected = parseFloat( $('.cart').find('input[name="quantity"]').val() );
					} else {
						var qty_selected = 1;
					}
					
					// Multiply booking price by quantity selected
					var new_price = wceb.formatPrice( this.fragments.booking_price * qty_selected );
					var price_html = '<span class="woocommerce-Price-amount amount">' + new_price + '</span>';

					// If the product is on sale
					if ( this.fragments.booking_regular_price !== '' ) {
						var new_regular_price = wceb.formatPrice( this.fragments.booking_regular_price * qty_selected );
						var price_html = '<del><span class="woocommerce-Price-amount amount">' + new_regular_price + '</span></del> <ins><span class="woocommerce-Price-amount amount">' + new_price + '</span></ins>';
						$('.booking_price').attr('data-booking_regular_price', this.fragments.booking_regular_price );
					}

					// Update price
					$('.booking_price').attr('data-booking_price', this.fragments.booking_price )
								       .find('.price').html( price_html );

					wceb.session = this.fragments.session;

				}

				$('body').trigger( 'update_price', [ data, response ] );

				// Unblock
				$('form.cart, form.bundle_form').stop(true).css('opacity', '1').unblock();
			
			});
		}
	}(jQuery));

});
