<?
/**
 * @Author: José Elías Gutierrez
 * @Date:   2017-04-10 10:58:40
 * @Last Modified by:   José Elías Gutierrez
 * @Last Modified time: 2017-04-21 14:17:11
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

function jebe_add_variation_price_fields($loop, $variation){
	$_booking_price_per_hour = get_post_meta($variation->ID, '_booking_price_per_hour', true);
	$_booking_price_per_day = get_post_meta($variation->ID, '_booking_price_per_day', true);
	$_booking_price_per_weekend = get_post_meta($variation->ID, '_booking_price_per_weekend', true);
	$_booking_price_per_holiday = get_post_meta($variation->ID, '_booking_price_per_holiday', true);
	require(dirname(__FILE__).'/views/variation_extra_fields.php');
}


function jebe_booking_price_fields_save($variation_id, $i){

	// Price per hour Field
	$booking_price_per_hour = $_POST['_var_booking_price_per_hour'][$i];
	if( !empty( $booking_price_per_hour ) )
		update_post_meta( $variation_id, '_booking_price_per_hour', esc_attr( $booking_price_per_hour ) );

	// Price per day Field
	$booking_price_per_day = $_POST['_var_booking_price_per_day'][$i];
	if( !empty( $booking_price_per_day ) )
		update_post_meta( $variation_id, '_booking_price_per_day', esc_attr( $booking_price_per_day ) );

	// Price per day Field
	$booking_price_per_weekend = $_POST['_var_booking_price_per_weekend'][$i];
	if( !empty( $booking_price_per_weekend ) )
		update_post_meta( $variation_id, '_booking_price_per_weekend', esc_attr( $booking_price_per_weekend ) );

	// Price per day Field
	$booking_price_per_holiday = $_POST['_var_booking_price_per_holiday'][$i];
	if( !empty( $booking_price_per_holiday ) )
		update_post_meta( $variation_id, '_booking_price_per_holiday', esc_attr( $booking_price_per_holiday ) );
}



function jebe_order_display_product_dates($item_id, $item, $_product ) {
	error_log('jebe_add_cart_item_data');
    global $wpdb;

    if ( ! isset( $item['product_id'] ) ) {
        return;
    }
    
    $product_id   = $item['product_id'];
    $variation_id = $item['variation_id'];

    $start_hour_set = wc_get_order_item_meta( $item_id, '_ebs_start_hour_display' );
    $end_hour_set   = wc_get_order_item_meta( $item_id, '_ebs_end_hour_display' );

    $start_hour_text = __( 'Start', 'jebe_easy_booking' );
    $end_hour_text   = __( 'End', 'jebe_easy_booking' );

    $item_order_meta_table = $wpdb->prefix . 'woocommerce_order_itemmeta';
    
    if ( ! empty( $start_hour_set ) ) {

        // Get meta ids from the database
        $start_display_meta_id = $wpdb->get_var( $wpdb->prepare(
            "SELECT `meta_id` FROM $item_order_meta_table WHERE `order_item_id` = %d AND `meta_key` LIKE %s",
            $item_id, '_ebs_start_hour_display'
        ));

        if ( ! empty( $end_hour_set ) ) {

            $end_display_meta_id = $wpdb->get_var( $wpdb->prepare(
                "SELECT `meta_id` FROM $item_order_meta_table WHERE `order_item_id` = %d AND `meta_key` LIKE %s",
                $item_id, '_ebs_end_hour_display'
            ));

        }

        // Product or variation ID
        $id      = empty( $variation_id ) ? $product_id : $variation_id;
        $product = wc_get_product( $id );

        $start_hour = $start_hour_set;
        $end_hour = $end_hour_set;

        include( 'views/order-item-meta.php' );

    }
}

class JEBE_Settings {

	public function __construct() {

		// get plugin options values
		$this->options = get_option('jebe_settings');
		$this->holidays = get_option('jebe_holidays');

		// initialize options the first time
		if ( ! $this->options ) {
		
		    $this->options = array(
		    	'jebe_default_hour_price'       => '',
		    	'jebe_default_day_price'        => '',
		    	'jebe_default_weekend_price'    => '',
		    	'jebe_default_holiday_price'	=> '',
		    	'jebe_allow_book_in_holiday'	=> 'off',
		    	'jebe_min_bookable_hour'		=> '0',
		    	'jebe_max_bookable_hour'		=> '0',
		    );

		    $this->holidays = array(
		    	'jebe_holidays'					=> array()
		    );

		    add_option( 'jebe_settings', $this->options );
		    add_option( 'jebe_holidays', $this->holidays );

		}

		if ( is_admin() ) {

			add_action( 'admin_menu', array( $this, 'jebe_add_admin_menu' ), 10 );

			add_action( 'admin_init', array( $this, 'jebe_settings_init' ) );
		}
	}

	public function jebe_settings_init( ) {

		add_action( 'jebe_settings_general_tab', array( $this, 'jebe_general_settings' ), 10);
		add_action( 'jebe_settings_holidays_tab', array( $this, 'jebe_holidays_settings' ), 10);

		add_action( 'jebe_settings_show_holidays', array( $this, 'jebe_holidays_dates_display' ), 10);

		register_setting(
			'jebe_general_settings',
			'jebe_settings', 
			array( $this, 'sanitize_values' )
		);

		add_settings_section(
			'jebe_general_main_settings',
			__( 'General settings', 'j_easy_booking_extended' ),
			array( $this, 'jebe_section_general' ),
			'jebe_general_settings'
		);

		add_settings_field(
			'jebe_global_hour_price',
			__( 'Default price per hour', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_default_hour_price' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);

		add_settings_field(
			'jebe_global_day_price',
			__( 'Default price per day', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_default_day_price' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);

		add_settings_field(
			'jebe_global_weekend_price',
			__( 'Default price per weekend', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_default_weekend_price' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);

		add_settings_field(
			'jebe_global_holiday_price',
			__( 'Default price per holiday', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_default_holiday_price' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);

		add_settings_field(
			'jebe_global_allow_book_in_holiday',
			__( 'Allow bookings in holidays', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_allow_book_in_holiday' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);

		add_settings_field(
			'jebe_min_bookable_hour',
			__( 'Min bookable hour', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_min_bookable_hour' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);

		add_settings_field(
			'jebe_max_bookable_hour',
			__( 'Max bookable hour', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_max_bookable_hour' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);


		// Holidays settings field

		register_setting(
			'jebe_holidays_settings',
			'jebe_holidays', 
			array( $this, 'sanitize_collection' )
		);

		add_settings_section(
			'jebe_holidays_main_settings',
			__( 'Holidays', 'j_easy_booking_extended' ),
			array( $this, 'jebe_section_holiday' ),
			'jebe_holidays_settings'
		);

		add_settings_field(
			'jebe_holiday',
			__( 'Add new holiday', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_holiday' ),
			'jebe_holidays_settings',
			'jebe_holidays_main_settings'
		);
		
		wp_enqueue_style('admin_css_bootstrap', plugins_url('/myplugin/css/bootstrap.min.css'), false, '1.0.0', 'all');

	}

	/**
    *
    * Add settings link
    *
    **/
    public function jebe_add_admin_menu( ) {
        add_submenu_page( 'easy-booking', 'Hour bookings', 'Hour bookings', 'manage_options', 'jebe_settings', array( $this, 'jebe_admin_page' ) );
    }

    public function jebe_admin_page( ) {
    	if ( !current_user_can( 'manage_options' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}

		include('views/settings.php');
    }

    public function jebe_general_settings( ) {
    	settings_fields( 'jebe_general_settings' );
		do_settings_sections( 'jebe_general_settings' );
    }

    public function jebe_holidays_settings( ) {
    	settings_fields( 'jebe_holidays_settings' );
		do_settings_sections( 'jebe_holidays_settings' );
    }

    public function jebe_holidays_dates_display( ) {
    	sort($this->holidays);
    	$holidays = $this->holidays;
    	include('views/holidays_list_view.php');
    }

    public function jebe_section_general( ) {
    	echo '';
    }

    public function jebe_section_holiday( ) {
    	echo '';
    }

    public function jebe_field_default_hour_price( ) {
    	wceb_settings_input( array(
			'type'              => 'number',
			'id'                => 'jebe_default_hour_price',
			'name'              => 'jebe_settings[jebe_default_hour_price]',
			'value'             => isset( $this->options['jebe_default_hour_price'] ) ? absint( $this->options['jebe_default_hour_price'] ) : '',
			'description'       => __('Default price per hour.', 'j_easy_booking_extended'),
			'custom_attributes' => array(
				'min' 	=> 0,
				'step' 	=> '.01'
			)
		));
    }

    public function jebe_field_default_day_price( ) {
    	wceb_settings_input( array(
			'type'              => 'number',
			'id'                => 'jebe_default_day_price',
			'name'              => 'jebe_settings[jebe_default_day_price]',
			'value'             => isset( $this->options['jebe_default_day_price'] ) ? absint( $this->options['jebe_default_day_price'] ) : '',
			'description'       => __('Default price per day.', 'j_easy_booking_extended'),
			'custom_attributes' => array(
				'min' 	=> 0,
				'step' 	=> '.01'
			)
		));
    }

    public function jebe_field_default_weekend_price( ) {
    	wceb_settings_input( array(
			'type'              => 'number',
			'id'                => 'jebe_default_weekend_price',
			'name'              => 'jebe_settings[jebe_default_weekend_price]',
			'value'             => isset( $this->options['jebe_default_weekend_price'] ) ? absint( $this->options['jebe_default_weekend_price'] ) : '',
			'description'       => __('Default price per weekend.', 'j_easy_booking_extended'),
			'custom_attributes' => array(
				'min' 	=> 0,
				'step' 	=> '.01'
			)
		));
    }

    public function jebe_field_default_holiday_price( ) {
    	wceb_settings_input( array(
			'type'              => 'number',
			'id'                => 'jebe_default_holiday_price',
			'name'              => 'jebe_settings[jebe_default_holiday_price]',
			'value'             => isset( $this->options['jebe_default_holiday_price'] ) ? absint( $this->options['jebe_default_holiday_price'] ) : '',
			'description'       => __('Default price per holiday.', 'j_easy_booking_extended'),
			'custom_attributes' => array(
				'min' 	=> 0,
				'step' 	=> '.01'
			)
		));
    }

    public function jebe_field_allow_book_in_holiday( ) {
    	wceb_settings_checkbox( array(
			'id'          => 'jebe_allow_book_in_holiday',
			'name'        => 'jebe_settings[jebe_allow_book_in_holiday]',
			'description' => __( 'Check to allow bookings in holidays.', 'j_easy_booking_extended' ),
			'value'       => isset( $this->options['jebe_allow_book_in_holiday'] ) ? $this->options['jebe_allow_book_in_holiday'] : '',
			'cbvalue'     => 'on'
		));
    }

    public function jebe_field_min_bookable_hour( ) {
    	$options = array();
    	foreach (range(0, 23) as $key => $value) {
    		$options[] = sprintf("%02d", $value).':00';
    	}

    	wceb_settings_select( array(
			'id'          => 'jebe_min_bookable_hour',
			'name'        => 'jebe_settings[jebe_min_bookable_hour]',
			'description' => __( 'Min bookable hour.', 'j_easy_booking_extended' ),
			'value'       => isset( $this->options['jebe_min_bookable_hour'] ) ? $this->options['jebe_min_bookable_hour'] : '',
			'options'	  => $options
		));
    }

    public function jebe_field_max_bookable_hour( ) {
    	$options = array();
    	foreach (range(0, 23) as $key => $value) {
    		$options[] = sprintf("%02d", $value).':00';
    	}

    	wceb_settings_select( array(
			'id'          => 'jebe_max_bookable_hour',
			'name'        => 'jebe_settings[jebe_max_bookable_hour]',
			'description' => __( 'Min bookable hour.', 'j_easy_booking_extended' ),
			'value'       => isset( $this->options['jebe_max_bookable_hour'] ) ? $this->options['jebe_max_bookable_hour'] : '',
			'options'	  => $options
		));
    }

    public function jebe_field_holiday( ) {
    	wceb_settings_input( array(
			'type'              => 'date',
			'id'                => 'jebe_holiday',
			'name'              => 'jebe_holiday',
			'value'             => '',
			'description'       => __('Add holiday date.', 'j_easy_booking_extended'),
			'custom_attributes' => array(),
			'class'				=> 'datepicker'
		));
    }

    public function sanitize_values( $settings ) {

    	return $settings;
    }

    public function sanitize_collection( $settings ) {
    	
    	if( isset( $_POST ) ) {


	        if( array_key_exists( 'jebe_holiday', $_POST ) )
	        	
	        	if( ! in_array( $_POST['jebe_holiday'], $this->holidays ) && ! ($_POST['jebe_holiday'] == '') )
		    	
		    		$this->holidays[] = $_POST['jebe_holiday'];

	    		if( isset( $_POST['delete'] ) ) {

	    			if( in_array( $_POST['delete'], $this->holidays ) )
			    		$this->holidays = array_diff( $this->holidays, array( $_POST['delete'] ) );

	    		}
		}

		$settings = $this->holidays;

    	return $settings;
    }

}

return new JEBE_Settings();