<?
/**
 * @Author: José Elías Gutierrez
 * @Date:   2017-04-11 09:54:15
 * @Last Modified by:   José Elías Gutierrez
 * @Last Modified time: 2017-04-11 18:07:05
 */

function jebe_check_hours_before_add_to_cart( $passed = true, $product_id, $quantity, $variation_id = '', $cart_item_data = array() ){

	error_log('jebe_check_hours_before_add_to_cart');
	
	$id = empty( $variation_id ) ? $product_id : $variation_id;
    if ( ! $passed ) {
        return $passed;
    }

    $product = wc_get_product( $id );

    if ( ! $product ) {
        return false;
    }
    
    // If product is bookable
    if ( wceb_is_bookable( $product ) ) {

        $booking_session = WC()->session->get( 'booking' );

        if ( isset( $booking_session[$id] ) && ! empty( $booking_session[$id] ) ) {
        	
            if ( ! isset( $booking_session[$id]['start_hour'] ) || ! isset( $booking_session[$id]['end_hour'] ) ) {
                wc_add_notice( __( 'Please choose hours', 'jebe_easy_booking' ), 'error' );
                $passed = false;
            }


        } else {
            wc_add_notice( __( 'Please choose hours', 'jebe_easy_booking' ), 'error' );
            $passed = false;

        }
    }
    return $passed;
}


/**
*
* Adds session data to cart item
*
* @param array $cart_item_meta
* @param int $product_id
* @return array $cart_item_meta
*
**/
function jebe_add_cart_item_data( $cart_item_meta, $product_id, $variation_id ) {
	error_log('jebe_add_cart_item_data');
    // Get session
    $booking_session = WC()->session->get( 'booking_hours' );
    $id              = empty( $variation_id ) ? $product_id : $variation_id;
    
    if ( isset( $booking_session[$id] ) && ! empty( $booking_session[$id] ) ) {

        if ( isset( $booking_session[$id]['start_hour'] ) ) {
            $cart_item_meta['_start_hour'] = sanitize_text_field( $booking_session[$id]['start_hour'] ); // Formatted dates
        }

        if ( isset( $booking_session[$id]['end_hour'] ) ) {
            $cart_item_meta['_end_hour'] = sanitize_text_field( $booking_session[$id]['end_hour'] ); // Formatted dates
        }

        // Reset session for this product ID
        unset( $booking_session[$id] );
        WC()->session->set( 'booking_hours', $booking_session );

    }
    
    return $cart_item_meta;
}


/**
*
* Adds data to cart item
*
* @param array $cart_item
* @param array $values - cart_item_meta
* @return array $cart_item
*
**/
function jebe_get_cart_item_from_session( $cart_item, $values ) {
	error_log('jebe_get_cart_item_from_session');
	error_log('jebe_get_cart_item_from_session: ' . $values['_start_hour'] );
    if ( isset( $values['_start_hour'] ) ) {
        $cart_item['_start_hour'] = $values['_start_hour'];
    }

    if ( isset( $values['_end_hour'] ) ) {
        $cart_item['_end_hour'] = $values['_end_hour'];
    }

    //$this->wceb_add_cart_item_booking_price( $cart_item );
    return $cart_item;
}


/**
*
* Adds formatted dates to the cart item
*
* @param array $other_data
* @param array $cart_item
* @return array $other_data
*
**/
function jebe_get_item_data( $other_data, $cart_item ) {
	error_log('jebe_get_item_data');
    // If is bundled, return
    if ( isset( $cart_item['bundled_by'] ) ) {
        return $other_data;
    }

    $product_id   = $cart_item['product_id'];
    $variation_id = $cart_item['variation_id'];

    $id = empty( $variation_id ) ? $product_id : $variation_id;

    $product = wc_get_product( $id );

    $start_hour_text = esc_html( __( 'Start hour', 'jebe_easy_booking' ) );
    $end_hour_text   = esc_html( __( 'End hour', 'jebe_easy_booking' ) );

    if ( isset( $cart_item['_start_hour'] ) && ! empty ( $cart_item['_start_hour'] ) ) {

        $other_data[] = array(
            'name'  => $start_hour_text,
            'value' => $cart_item['_start_hour']
        );

    }

    if ( isset( $cart_item['_end_hour'] ) && ! empty ( $cart_item['_end_hour'] ) ) {

        $other_data[] = array(
            'name'  => $end_hour_text,
            'value' => $cart_item['_end_hour']
        );

    }

    return $other_data;
}


/**
*
* Deprecated in WooCommerce 2.7
* Adds booked dates to the order item
*
* @param int $item_id
* @param array $values - 
*
**/
function jebe_add_order_meta( $item_id, $values ) {
    error_log('jebe_add_order_meta');
    if ( ! empty( $values['_start_hour'] ) ) {
        wc_add_order_item_meta( $item_id, '_ebs_start_hour_display', sanitize_text_field( $values['_start_hour'] ) );
    }

    if ( ! empty( $values['_end_hour'] ) ) {
        wc_add_order_item_meta( $item_id, '_ebs_end_hour_display', sanitize_text_field( $values['_end_hour'] ) );
    }

}

/**
*
* Since WooCommerce 2.7
* Adds booked dates to the order item
*
* @param int $item_id
* @param array $values - 
*
**/
function jebe_order_line_item( $item, $cart_item_key, $values ) {
	error_log('jebe_order_line_item');
    if ( ! empty( $values['_start_hour'] ) ) {
        $item->add_meta_data( '_ebs_start_hour_display', sanitize_text_field( $values['_start_hour'] ) );
    }

    if ( ! empty( $values['_end_hour'] ) ) {
        $item->add_meta_data( '_ebs_end_hour_display', sanitize_text_field( $values['_end_hour'] ) );
    }

}


/**
*
* Deprecated in WooCommerce 2.7
* Display order item booking meta (start and (maybe) end date(s)), even if hidden
*
* @param str $output
* @param WC_Order_Item $order_item
* @param str $output
*
**/
function jebe_display_booking_item_meta( $output, $order_item ) {
	error_log('jebe_display_booking_item_meta');
    $order_item_meta = $order_item->meta;
    $product = $order_item->product;

    $start_text = esc_html( __( 'Start hour', 'jebe_easy_booking' ) );
    $end_text   = esc_html( __( 'End hour', 'jebe_easy_booking' ) );

    if ( isset( $order_item_meta['_ebs_start_hour_display'] ) ) {

        foreach ( $order_item_meta['_ebs_start_hour_display'] as $index => $meta ) {
            $output .= '<dl class="variation">' . wp_kses_post( $start_text . ': ' . $meta ) . '</dl>';
        }

    }

    if ( isset( $order_item_meta['_ebs_end_hour_display'] ) ) {

        foreach ( $order_item_meta['_ebs_end_hour_display'] as $index => $meta ) {
            $output .= '<dl class="variation">' . wp_kses_post( $end_text . ': ' . $meta ) . '</dl>';
        }

    }
    
    return $output;

}

/**
*
* Since WooCommerce 2.7
* Display order item booking meta (start and (maybe) end date(s)), even if hidden
*
* @param str $output
* @param WC_Order_Item $order_item
* @param str $output
*
**/
function jebe_display_item_meta( $html, $item, $args ) {
	error_log('jebe_display_item_meta');
    $product = is_callable( array( $item, 'get_product' ) ) ? $item->get_product() : $item->product;

    $start_text = esc_html( __( 'Start', 'jebe_easy_booking' ) );
    $end_text   = esc_html( __( 'End', 'jebe_easy_booking' ) );

    $start_hour = $item->get_meta( '_ebs_start_hour_display' );
    if ( isset( $start_hour ) ) {
        $html .= '<dl class="variation">' . wp_kses_post( $start_text . ': ' . $start_hour ) . '</dl>';
    }

    $end_hour = $item->get_meta( '_ebs_end_hour_display' );
    if ( isset( $end_hour ) ) {
        $html .= '<dl class="variation">' . wp_kses_post( $end_text . ': ' . $end_hour ) . '</dl>';
    }

    return $html;
}


/**
*
* Hides dates on the order page (to display a custom form instead)
*
* @param array $item_meta - Hidden values
* @return array $item_meta
*
**/
function jebe_hide_formatted_date( $item_meta ) {

    $item_meta[] = '_ebs_start_hour_display';
    $item_meta[] = '_ebs_end_hour_display';

    return $item_meta;

}