<?php
/**
 * Plugin Name: Easy Booking Hour Extended
 * Description: WooCommerce Easy Booking add-on for hour bookings, stocks and availability
 * Version: 1.0.0
 * Text domain: j_easy_booking_extended
 * Licence : GPLv2 or later
 * @Author: José Elías Gutierrez
 * @Date:   2017-04-10 10:51:30
 * @Last Modified by:   José Elías Gutierrez
 * @Last Modified time: 2017-04-21 14:10:03
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

if ( ! class_exists( 'J_Easy_Booking_Extended' ) ) :

class J_Easy_Booking_Extended {

    protected static $_instance = null;

    public static function instance() {

        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    public function __construct() {

        // Check if WooCommerce Easy Booking is active
        if ( $this->jebe_easy_booking_is_active() ) {

            $plugin = plugin_basename( __FILE__ );

            // Init plugin
            add_action( 'plugins_loaded', array( $this, 'jebe_init' ), 20 );

            # TODO: move actions to proper function to avoid unused functions querying

            // Add custom fields to product variations (price per hour, price per day, price per weekend, holiday price)
            add_action( 'easy_booking_after_variation_booking_options', 'jebe_add_variation_price_fields', 10, 2 );


            //  Save variations price fields
            add_action( 'woocommerce_save_product_variation', 'jebe_booking_price_fields_save', 10, 2 );

            add_action('woocommerce_before_add_to_cart_button', array( $this, 'remove_actions' ), 1 );
            add_action('woocommerce_before_add_to_cart_button', 'jebe_display_for_variable_product', 20 );


            // Product page actions

            # wceb returns html. We need to override it.
            $this->remove_ajax_actions();
            
            add_action( 'wp_ajax_add_new_price', array( $this,  'jebe_get_new_price' ) );
            add_action( 'wp_ajax_nopriv_add_new_price', array( $this, 'jebe_get_new_price' ) );

            # Cart actions
            add_filter( 'woocommerce_add_to_cart_validation', 'jebe_check_hours_before_add_to_cart', 30, 5 );
            add_filter( 'woocommerce_add_cart_item_data', 'jebe_add_cart_item_data', 30, 3 );
            add_filter( 'woocommerce_get_cart_item_from_session', 'jebe_get_cart_item_from_session' , 99, 2 );
            add_filter( 'woocommerce_get_item_data', 'jebe_get_item_data' , 20, 2 );

            # checkout actions
            if ( version_compare( WC_VERSION, '2.7', '<' ) ) {
	            add_action( 'woocommerce_add_order_item_meta', 'jebe_add_order_meta', 10, 2 );
	            add_filter( 'woocommerce_order_items_meta_display', 'jebe_display_booking_item_meta', 10, 2 );
        	} else {
	            add_action( 'woocommerce_checkout_create_order_line_item', 'jebe_order_line_item', 20, 3 );
	            add_filter( 'woocommerce_display_item_meta', 'jebe_display_item_meta', 20, 3 );
	        }
            add_filter( 'woocommerce_hidden_order_itemmeta', 'jebe_hide_formatted_date', 20, 1 );

            # display hours in order page
            add_action( 'woocommerce_before_order_itemmeta', 'jebe_order_display_product_dates', 20, 3 ); 

        }

    }

    public function get_max_bookable_qty( $variation_id, $data ) {

    	$bookable = true;

    	$variation_obj = new WC_Product_variation($variation_id);
        $stock = $variation_obj->get_stock_quantity();

    	$start_date = new DateTime( $data['start'] . ' ' . $data['start_hour'] );
    	$end_date 	= new DateTime( $data['end'] . ' ' . $data['end_hour'] );

    	$bookings = $this->get_variation_bookings( $variation_id );

    	$overlap = array();

    	// check if booking dont overlap 
    	foreach ($bookings['booked'] as $key => $booking) {
			
			$booking_start_date = new DateTime( $booking['start_date'] . ' ' . $booking['start_hour'] );
    		$booking_end_date 	= new DateTime( $booking['end_date'] . ' ' . $booking['end_hour'] );
    		
    		error_log( $end_date->format('Y-m-d H:i') . " < " . $booking_start_date->format('Y-m-d H:i') . " || " . $start_date->format('Y-m-d H:i') . " > " . $booking_end_date->format('Y-m-d H:i') );
    		
    		if( !( $end_date < $booking_start_date || $start_date > $booking_end_date ) ){
    			$overlap[] = $booking;
    		}

    	}

    	// no overlap, so all stock is bookable
    	if(count($overlap) == 0)
    		return $stock;

    	// overlaps, let's check qtys
		// make 2 hour ranges as it's the minimun (maybe moved to settings) and check for everyone. If all of them has qty available, is bookable.
		$interval = new DateInterval('PT2H');
		$daterange = new DatePeriod($start_date, $interval ,$end_date);
    	
    	$booked_per_hour = array();

    	foreach ($daterange as $key => $date) {
    		
    		foreach ($overlap as $key => $booking) {
    			$booking_start_date = new DateTime( $booking['start_date'] . ' ' . $booking['start_hour'] );
    			$booking_end_date 	= new DateTime( $booking['end_date'] . ' ' . $booking['end_hour'] );	
    			
    			if( $date >= $booking_start_date && $date <= $booking_end_date){
    			
    				$date_key = $date->format('Y-m-d H:i');
    				if ( ! array_key_exists( $date_key, $booked_per_hour ) )
    					$booked_per_hour[$date_key] = 0;		
    			
    				$booked_per_hour[$date_key] += $booking['qty'];
    			}
    		}
    	}

    	// seek and return the minimun available qty
        $available = $stock;
    	foreach ($booked_per_hour as $key => $booked) {
    		$diff = $stock - $booked;
    		
    		if($diff < $available){
				$available = $diff;
    		}
    	}

    	return $available;
    }


    public function get_variation_bookings( $variation_id ) {
		global $wpdb;

    	$variation = wc_get_product( $variation_id );
        
        $data = array('variation_id' => $variation_id,
        	'booked' => array());

        if ( ! wceb_is_bookable( $variation ) ) {
            return $data;
        }

        $item_ids = $wpdb->get_results( $wpdb->prepare( "
        	SELECT order_item_id 
			FROM " . $wpdb->prefix . "woocommerce_order_itemmeta 
			WHERE meta_key='_variation_id' 
				AND meta_value=%s 
				AND order_item_id IN ( SELECT order_item_id 
										FROM " . $wpdb->prefix . "woocommerce_order_itemmeta 
										WHERE meta_key='_ebs_end_format' 
											AND meta_value>=%s )
			", $variation_id, date('Y-m-d'))
        );


        foreach ($item_ids as $key => $item) {
			$data['booked'][] = array(
                'start_hour'	=> wc_get_order_item_meta( $item->order_item_id, '_ebs_start_hour_display', true ),
                'end_hour'		=> wc_get_order_item_meta( $item->order_item_id, '_ebs_end_hour_display', true ),
				'start_date'	=> wc_get_order_item_meta( $item->order_item_id, '_ebs_start_format', true ),
				'end_date'		=> wc_get_order_item_meta( $item->order_item_id, '_ebs_end_format', true ),
				'qty'			=> wc_get_order_item_meta( $item->order_item_id, '_qty', true ),
			);
        }

        return $data;
    }

    public function jebe_get_config_data( ) {
		global $post;
    	
    	$options = get_option('jebe_settings');
		$holidays = get_option('jebe_holidays');

		sort($holidays);
		$today = date('Y-m-d');
		$holidays = array_filter($holidays, function($value){
			return $value >= date('Y-m-d');
		});

		// get hours not available for current variations

        $availabilities = array();

        // If not on a single product page, return
        if ( is_product() ) {
	        
	        $product = wc_get_product( $post->ID );

	        if ( $product ) {
		        // Load scripts only on product page if "booking" option is checked
	        	if ( wceb_is_bookable( $product ) ) {
		        
		            // WooCommerce 2.7 compatibility
		            if ( is_callable( array( $product, 'get_type' ) ) ) {
		                $product_type = $product->get_type();
		            } else {
		                $product_type = $product->product_type;
		            }    
		        }

                // This should only work with variations, so this is ok
                // other wise, we must take care of different product types
		        $variation_ids = $product->get_children();

		        if ( $variation_ids ) {

                    foreach ( $variation_ids as $variation_id ) {

                        $data = $this->get_variation_bookings($variation_id);

                        $availabilities[$variation_id] = $data;

                    }
                }
		    }
        }


		return array('allow_holiday_bookings' =>  $options['jebe_allow_book_in_holiday'],
			'holidays' => $holidays,
			'availabilities' => $availabilities,
			'options' => $options);
    }

    public function jebe_get_new_price( ) {

        check_ajax_referer( 'set-dates', 'security' );

        $options = get_option('jebe_settings');

        $min_bookable_hour = sprintf("%02d", $options['jebe_min_bookable_hour']) . ':00';
        $max_bookable_hour = sprintf("%02d", $options['jebe_max_bookable_hour']) . ':00';

        $product_id   = isset( $_POST['product_id'] ) ? absint( $_POST['product_id'] ) : ''; // Product ID
        $variation_id = isset( $_POST['variation_id'] ) ? absint( $_POST['variation_id'] ) : ''; // Variation ID
        $children     = isset( $_POST['children'] ) ? array_map( 'absint', $_POST['children'] ) : array(); // Product children for grouped and variable products

        $id = ! empty( $variation_id ) ? $variation_id : $product_id; // Product or variation id

        $calc_mode = $this->options['easy_booking_calc_mode']; // Calculation mode (Days or Nights)

        $start_date = isset( $_POST['start'] ) ? sanitize_text_field( $_POST['start'] ) : ''; // Booking start date
        $end_date   = isset( $_POST['end'] ) ? sanitize_text_field( $_POST['end'] ) : ''; // Booking end date

        $start = isset( $_POST['start_format'] ) ? sanitize_text_field( $_POST['start_format'] ) : ''; // Booking start date 'yyyy-mm-dd'
        $end   = isset( $_POST['end_format'] ) ? sanitize_text_field( $_POST['end_format'] ) : ''; // Booking end date 'yyyy-mm-dd'

        $start_hour = isset( $_POST['start_hour'] ) 
        	? (( $_POST['start_hour'] != '' ) ? sanitize_text_field( $_POST['start_hour'] ) : $min_bookable_hour) 
        	: $min_bookable_hour; // Booking start date. Defaults to 8:00 if not present
        $end_hour   = isset( $_POST['end_hour'] ) 
        	? (( $_POST['end_hour'] != '' ) ? sanitize_text_field( $_POST['end_hour'] ) : $max_bookable_qty)
        	: $max_bookable_hour; // Booking end date. Defaults to 20:00 if not present

        $product  = wc_get_product( $product_id ); // Product object
        $_product = ( $product_id !== $id ) ? wc_get_product( $id ) : $product; // Product or variation object


        // variation booking prices. This should be populated with defaults from admin. 

        $default_prices = get_option('jebe_settings');
        $allow_book_in_holidays = $default_prices['jebe_allow_book_in_holiday'];

        $booking_prices = [
        	'hour' 		=> $default_prices['jebe_default_hour_price'],
        	'day' 		=> $default_prices['jebe_default_day_price'],
        	'weekend' 	=> $default_prices['jebe_default_weekend_price'],
        	'holiday' 	=> $default_prices['jebe_default_holiday_price']
        ];

        if ( version_compare( WC_VERSION, '2.7', '<' ) ) {
        	$_prices['hour']	 = get_post_meta( $id, '_booking_price_per_hour', true );
        	$_prices['day']		 = get_post_meta( $id, '_booking_price_per_day', true );
        	$_prices['weekend']	 = get_post_meta( $id, '_booking_price_per_weekend', true );
        	$_prices['holiday']	 = get_post_meta( $id, '_booking_price_per_holiday', true );
	    } else {
	    	$_prices['hour']	 = $_product->get_meta( '_booking_price_per_hour', true );
        	$_prices['day']		 = $_product->get_meta( '_booking_price_per_day', true );
        	$_prices['weekend']	 = $_product->get_meta( '_booking_price_per_weekend', true );
        	$_prices['holiday']	 = $_product->get_meta( '_booking_price_per_holiday', true );
	    }

	    $booking_prices['hour'] = $_prices['hour'] > 0 ? $_prices['hour'] : $booking_prices['hour'] ;
	    $booking_prices['day'] = $_prices['day'] > 0 ? $_prices['day'] : $booking_prices['day'] ;
	    $booking_prices['weekend'] = $_prices['weekend'] > 0 ? $_prices['weekend'] : $booking_prices['weekend'] ;
	    $booking_prices['holiday'] = $_prices['holiday'] > 0 ? $_prices['holiday'] : $booking_prices['holiday'] ;


        if ( ! $_product ) {
            return;
        }

        // If product is variable and no variation was selected
        if ( $product->is_type( 'variable' ) && empty( $variation_id ) ) {
            $this->wceb_throw_error( 3 );
        }

        // If product is grouped and no quantity was selected for grouped products
        if ( $product->is_type( 'grouped' ) && empty( $children ) ) {
            $this->wceb_throw_error( 4 );
        }

        $number_of_dates = wceb_get_product_booking_dates( $_product );

        // If date format is "one", check only one date is set
        if ( $number_of_dates === 'one' ) {
            
            $dates = 'one_date';
            $duration = 1;

            // If end date is set
            if ( ! empty( $end_date ) || ! empty( $end ) ) {
                $this->wceb_throw_error( 5 );
            }

            // If date is empty
            if ( empty( $start_date ) || empty( $start ) ) {
                $this->wceb_throw_error( 6 );
            }

        } else { // "Two" dates check

            $dates = 'two_dates';

            // If one date is empty
            if ( empty( $start_date ) || empty( $end_date ) || empty( $start ) || empty( $end ) ) {
                $this->wceb_throw_error( 2 );
            }

            $start_time = strtotime( $start );
            $end_time   = strtotime( $end );

            // If end date is before start date
            if ( $end_time < $start_time ) {
                $this->wceb_throw_error( 1 );
            }

            // Get booking duration in days
            $duration = absint( ( $start_time - $end_time ) / 86400 );

            if ( $duration == 0 ) {
                $duration = 1;
            }

            // If booking mode is days and calculation mode is set to "Days", add one day
            if ( $calc_mode === 'days' && ( $start != $end ) ) {
                $duration += 1 ;
            }

            $booking_duration = wceb_get_product_booking_duration( $_product );

            // If booking mode is weeks and duration is a multiple of 7
            if ( $booking_duration === 'weeks' ) {

                if ( $calc_mode === 'nights' && $duration % 7 === 0 ) { // If in weeks mode, check that the duration is a multiple of 7
                    $duration /= 7;
                } else if ( $calc_mode === 'days' && $duration % 6 === 0 ) { // Or 6 in "Days" mode
                    $duration /= 6;
                } else { // Otherwise throw an error
                    $this->wceb_throw_error( 1 );
                }
                
            } else if ( $booking_duration === 'custom' ) {

                $custom_booking_duration = wceb_get_product_custom_booking_duration( $_product );

                if ( $duration % $custom_booking_duration === 0 ) {
                    $duration /= $custom_booking_duration;
                } else {
                    $this->wceb_throw_error( 1 );
                }

            }

            // If number of days is inferior to 0
            if ( $duration <= 0 ) {
                $this->wceb_throw_error( 1 );
            }

        }

        // Get additional costs (for WooCommerce Product Addons)
        $additional_cost = $this->jebe_get_additional_costs( $_product );

        // Store data in array
        $data = array(
            'start_date' => $start_date,
            'start'      => $start
        );

        if ( isset( $duration ) && ! empty( $duration ) ) {
            $data['duration'] = $duration;
        }

        if ( isset( $end_date ) && ! empty( $end_date ) ) {
            $data['end_date'] = $end_date;
        }

        if ( isset( $end ) && ! empty( $end ) ) {
            $data['end'] = $end;
        }

        $booking_data = array();

        $new_price = 0;
        $new_regular_price = 0;

        // Grouped or Bundle product types
        if ( $product->is_type( 'grouped' ) || $product->is_type( 'bundle' ) ) {

            if ( ! empty( $children ) ) foreach ( $children as $child_id => $quantity ) {

                if ( $quantity <= 0 || ( $child_id === $id ) ) {
                    continue;
                }

                $child = wc_get_product( $child_id );

                $children_prices[$child_id] = (array) wceb_get_product_price( $product, $child, false, 'array' );

                // Multiply price by duration only if children is bookable
                if ( $children_prices[$child_id] ) {

                    if ( wceb_is_bookable( $child ) ) {

                        if ( $children_prices[$child_id] ) foreach ( $children_prices[$child_id] as $price_type => $price ) {

                            if ( $price === "" ) {
                                continue;
                            }

                            if ( $number_of_dates === 'two' ) {
                                $price *= $duration;
                            }

                            ${'child_new_' . $price_type} = apply_filters(
                                'easy_booking_' . $dates . '_price',
                                wc_format_decimal( $price ), // Regular or sale price for x days
                                $product, $child, $data, $price_type
                            );

                        }

                    } else {

                        $child_new_price = wc_format_decimal( $children_price[$child_id]['price'] );

                        if ( isset( $children_price[$child_id]['regular_price'] ) ) {
                            $child_new_regular_price = wc_format_decimal( $children_price[$child_id]['regular_price'] );
                        }

                    }

                } else {

                    // Tweak for not individually sold bundled products
                    $child_new_price = 0;
                    $child_new_regular_price = 0;

                }

                // Maybe add additional costs
                if ( isset( $additional_cost[$child_id] ) ) {

                    $child_new_price = $this->wceb_add_additional_costs( $child_new_price, $additional_cost[$child_id], $duration );

                    if ( isset( $child_new_regular_price ) ) {
                        $child_new_regular_price = $this->wceb_add_additional_costs( $child_new_regular_price, $additional_cost[$child_id], $duration );
                    }
                    
                }

                $data['new_price'] = $child_new_price;

                if ( isset( $child_new_regular_price ) && ! empty( $child_new_regular_price ) ) {
                    $data['new_regular_price'] = $child_new_regular_price;
                }

                // Store parent produt for bundled items
                if ( $product->is_type( 'bundle' ) ) {
                    $data['grouped_by'] = $product;
                }

                $booking_data[$child_id] = $data;

                if ( $product->is_type( 'grouped' ) ) {
                    $new_price += wc_format_decimal( $child_new_price * $quantity );

                    if ( isset( $child_new_regular_price ) ) {
                        $new_regular_price += wc_format_decimal( $child_new_regular_price * $quantity );
                    }
                }

            }

            if ( $product->is_type( 'bundle' ) ) {

                $prices = (array) wceb_get_product_price( $product, false, false, 'array' );

                if ( $prices ) foreach ( $prices as $price_type => $price ) {

                    if ( $price === "" ) {
                        continue;
                    }

                    if ( $number_of_dates === 'two' ) {
                        $price *= $duration;
                    }

                    ${'new_' . $price_type} = apply_filters(
                        'easy_booking_' . $dates . '_price',
                        wc_format_decimal( $price ), // Regular or sale price for x days
                        $product, $_product, $data, $price_type
                    );

                    if ( isset( $additional_cost[$id] ) && $additional_cost[$id] > 0 ) {
                        ${'new_' . $price_type} = $this->wceb_add_additional_costs( ${'new_' . $price_type}, $additional_cost[$id], $duration );
                    }

                }
                
            }

        } else {

            // Get product price and (if on sale) regular price
            $prices = (array) wceb_get_product_price( $_product, false, false, 'array' );

            if ( $prices ) foreach ( $prices as $price_type => $price ) {

                if ( $price === "" ) {
                    continue;
                }

                if ( $number_of_dates === 'two' ) {
                    $price *= $duration;
                }

                ${'new_' . $price_type} = apply_filters(
                    'easy_booking_' . $dates . '_price',
                    wc_format_decimal( $price ), // Regular or sale price for x days
                    $product, $_product, $data, $price_type
                );

                if ( isset( $additional_cost[$id] ) && $additional_cost[$id] > 0 ) {
                    ${'new_' . $price_type} = $this->jebe_add_additional_costs( ${'new_' . $price_type}, $additional_cost[$id], $duration );
                }

            }
            
        }


        // Hour based price calculation
        if( wceb_is_bookable( $product ) ) {

    		$start_time = DateTime::createFromFormat('H:i Y-m-d', $start_hour . ' ' . $start );
    		$end_time = DateTime::createFromFormat('H:i Y-m-d', $end_hour . ' ' .$end );

        	// Same day. Hour rate 
        	if( $duration == 1 && ( $start == $end ) ) {

        		if( ! empty( $start_hour ) && ! empty( $end_hour ) ) {

	        		$diff_time = $end_time->diff($start_time);
	        		$booking_duration = ($diff_time->days * 24) + $diff_time->h + ($diff_time->i / 60) + ($diff_time->s / 3600);
	        		$new_price = $booking_duration * $booking_prices['hour'];
        			
        		}
        	// Full day. Day rate
        	} else if( $duration == 1 ) {

        		$new_price = $duration * $booking_prices['day'];

        	// 2+ days. 
        	} else {

        		if( jebe_booking_is_weekend( $start_time, $end_time ) ) {

        			$new_price = $booking_prices['weekend'];
        		
        		} else {
        			// day rate
        			$new_price = $duration * $booking_prices['day'];
        		}

        	}
        }

        $data['new_price'] = $new_price;

        if ( isset( $new_regular_price ) && ! empty( $new_regular_price ) && ( $new_regular_price !== $new_price ) ) {
            $data['new_regular_price'] = $new_regular_price;
        } else {
            unset( $data['new_regular_price'] ); // Unset value in case it was set for a child product
        }
        $data['start_hour'] = $start_hour;
        $data['end_hour'] = $end_hour;

        $data['booking_prices'] = $booking_prices;

        $data['fragments'] = apply_filters( 'easy_booking_fragments', array(
                'session'               => true,
                'booking_price'         => esc_attr( $new_price ),
                'booking_regular_price' => isset( $new_regular_price ) ? esc_attr( $new_regular_price ) : '',
                'input.wceb_nonce'      => '<input type="hidden" name="_wceb_nonce" class="wceb_nonce" value="' . wp_create_nonce( 'set-dates' ) . '">'
                )
            );

        if ( isset( $details ) ) {
            $data['fragments']['p.booking_details'] = wp_kses_post( $details );
        }

        $data['max_bookable_qty'] = $this->get_max_bookable_qty( $variation_id, $data );

        $booking_data[$id] = $data;


        // Update session data
        if ( ! WC()->session->has_session() ) {
            WC()->session->set_customer_session_cookie( true );
        }

        WC()->session->set( 'booking', $booking_data );
        WC()->session->set( 'booking_hours', $booking_data );

        wp_send_json( $data );
        die();
    }

    /**
    *
    * Adds additional costs (compatibility with WooCommerce Product Addons)
    * @param int - $price - Product price
    * @param int - $additional_cost
    * @param int - $duration
    * @return int - $price
    *
    **/
    private function jebe_add_additional_costs( $price, $additional_cost, $duration ) {

        // Pass true to filter to multiply additional costs by booking duration (default: false)
        if ( true === apply_filters( 'easy_booking_multiply_additional_costs', false ) ) {
            $price += ( $additional_cost * $duration );
        } else {
            $price += $additional_cost; 
        }

        return $price;
    }

        /**
    *
    * Formats additional costs (compatibility with WooCommerce Product Addons)
    * @param Wc_Product - $_product - Product or variation object
    * @return array - $additional_cost
    *
    **/
    private function jebe_get_additional_costs( $_product ) {

        // Get additional costs
        $additional_cost = isset( $_POST['additional_cost'] ) ? array_map( 'wc_format_decimal', $_POST['additional_cost'] ) : array();
            
        $prices_include_tax = get_option( 'woocommerce_prices_include_tax' );
        $tax_display_mode   = get_option( 'woocommerce_tax_display_shop' );
        
        // Get additional costs including or excluding taxes (for WooCommerce Product Addons)
        if ( ! empty( $additional_cost ) ) {

            foreach ( $additional_cost as $ac_id => $ac_amount ) {

                if ( $_product->is_taxable() ) {

                    $rates = WC_Tax::get_base_tax_rates( $_product->get_tax_class() );

                    if ( $prices_include_tax === 'yes' && $tax_display_mode === 'excl' ) {

                        $taxes = WC_Tax::calc_exclusive_tax( $ac_amount, $rates );

                        if ( $taxes ) foreach ( $taxes as $tax ) {
                            $additional_cost[$ac_id] += $tax;
                        }

                    } else if ( $prices_include_tax === 'no' && $tax_display_mode === 'incl' ) {

                       $taxes = WC_Tax::calc_inclusive_tax( $ac_amount, $rates );

                       if ( $taxes ) foreach ( $taxes as $tax ) {
                            $additional_cost[$ac_id] -= $tax;
                        }

                    }

                }

            }
            
        }

        return $additional_cost;
    }

    public function remove_ajax_actions(){
	    global $wp_filter;
	    $tag = 'wp_ajax_add_new_price';
	    $priority = 10;
	    $tag_hooks = $wp_filter[ $tag ]->callbacks[ $priority ];
	    if(is_null($tag_hooks)) return;
	    foreach ( $tag_hooks as $the_tag => $the_callback ) {
	    	if($the_callback['function'][1] == 'wceb_get_new_price'){
	        	remove_action( $tag, $the_callback[ 'function' ], $priority );
	      	}
	    }
	}

    public function remove_actions(){
	    global $wp_filter;
	    $tag = 'woocommerce_single_variation';
	    $priority = 10;
	    $tag_hooks = $wp_filter[ $tag ]->callbacks[ $priority ];
	    if(is_null($tag_hooks)) return;
	    foreach ( $tag_hooks as $the_tag => $the_callback ) {
	    	if($the_callback['function'][1] == 'wceb_before_add_to_cart_button'){
	        	remove_action( $tag, $the_callback[ 'function' ], $priority );
	      	}
	    }
	    $tag = 'woocommerce_before_add_to_cart_button';
	    $priority = 20;
	    $tag_hooks = $wp_filter[ $tag ]->callbacks[ $priority ];
	    if(is_null($tag_hooks)) return;
	    foreach ( $tag_hooks as $the_tag => $the_callback ) {
	    	if($the_callback['function'][1] == 'wceb_before_add_to_cart_button'){
	        	remove_action( $tag, $the_callback[ 'function' ], $priority );
	      	}
	    }
	}
    /**
    *
    * Check if WooCommerce Easy Booking is active
    *
    * @return bool
    *
    **/
    private function jebe_easy_booking_is_active() {

        // Get active plugins
        $active_plugins = (array) get_option( 'active_plugins', array() );

        if ( is_multisite() ) {
            $active_plugins = array_merge( $active_plugins, get_site_option( 'active_sitewide_plugins', array() ) );
        }

        // Return true if Easy Booking is active
        return ( array_key_exists( 'j-easy-booking-extended/j-easy-booking-extended.php', $active_plugins ) || in_array( 'j-easy-booking-extended/j-easy-booking-extended.php', $active_plugins ) );

    }

    /**
    *
    * Init plugin
    *
    **/
    public function jebe_init() {

        // Don't init plugin if WooCommerce Easy Booking isn't at least version 1.9.
        if ( ! method_exists( WCEB(), 'wceb_get_version' ) ) {
            return;
        }
        
        // Define constants
        $this->jebe_define_constants();

        // load plugin textdomain
        $this->jebe_load_textdomain();

        // Common includes
        $this->jebe_includes();

        // Admin includes
        if ( is_admin() ) {
            $this->jebe_admin_includes(); 
        }
        
        // Frontend includes
        if ( ! is_admin() || defined( 'DOING_AJAX' ) ) {
            $this->jebe_frontend_includes();
        }

    }

    /**
    *
    * Define constants
    *
    **/
    private function jebe_define_constants() {
        // Plugin directory path
        define( 'JEBE_PLUGIN_FILE', __FILE__ );
    }

    /**
    *
    * Load plugin textdomain for translations
    *
    **/
    private function jebe_load_textdomain() {
        load_plugin_textdomain( 'j_easy_booking_extended', false, basename( dirname(__FILE__) ) . '/languages/' );
    }

    /**
    *
    * Common includes
    *
    **/
    private function jebe_includes() {
    	include_once( 'core/jebe-functions.php' );
    	include_once( 'core/front/cart.php' );
        /*include_once( 'includes/jebe-functions.php' );
        include_once( 'includes/jebe-date-functions.php' );
        include_once( 'includes/jebe-product-functions.php' );
        include_once( 'includes/class-jebe-ajax.php' );
        include_once( 'includes/class-jebe-cart.php' );
        include_once( 'includes/admin/class-jebe-order.php' );*/
    }

    /**
    *
    * Admin includes
    *
    **/
    private function jebe_admin_includes() {
    	include_once( 'core/admin.php' );
    	/*
        include_once( 'includes/admin/class-jebe-admin-assets.php' );
        include_once( 'includes/settings/class-jebe-settings.php' );
        include_once( 'includes/imports/class-jebe-list-imports.php' );
        include_once( 'includes/imports/class-jebe-imports.php' );
        include_once( 'includes/reports/class-jebe-reports.php' );   
        */
    }

    /**
    *
    * Frontend includes
    *
    **/
    private function jebe_frontend_includes() {
    	include_once( 'core/front/functions.php' );
    	include_once( 'core/front/cart.php' );

        //include_once( 'includes/class-jebe-product-view.php' );
        //include_once( 'includes/class-jebe-assets.php' );
    }

    /**
    *
    * Add notices
    *
    **/
    public function jebe_add_notices() {

        if ( is_multisite() ) {
            return;
        }

        if ( ! method_exists( WCEB(), 'wceb_get_version' ) ) {
            include_once( 'includes/admin/views/html-jebe-notice-update.php' );
            return;
        }

        $settings    = get_option('easy_booking_availability_settings');
        $license_key = isset( $settings['easy_booking_availability_license_key'] ) ? $settings['easy_booking_availability_license_key'] : false;

        if ( empty( $license_key ) ) {
            $license_key = false;
        }

        $screen = get_current_screen();

        // Don't display notices on the plugin settings page
        if ( in_array( $screen->id, array( 'easy-booking_page_easy-booking-availability-check' ) ) ) {
            return;
        }

        // Display a notice to init availabilities
        if ( get_option( 'easy_booking_display_notice_init' ) !== '1' ) {
            include_once( 'includes/admin/views/html-jebe-notice-init.php' );
        }

        if ( get_option( 'easy_booking_display_notice_init' ) == '1' && get_option( 'easy_booking_display_notice_init_after_update' ) !== '1' ) {
            include_once( 'includes/admin/views/html-jebe-notice-init-after-update.php' );
        }

        // Display license key notice
        if ( ! is_multisite() && get_option( 'easy_booking_display_notice_jebe_license' ) !== '1' && ! $license_key ) {
            include_once( 'includes/admin/views/html-jebe-notice-license-key.php' );
        }
    }

}

function JEBE() {
    return J_Easy_Booking_Extended::instance();
}

JEBE();

endif;